using System;
using System.Text.RegularExpressions;

// Each method of the Validate class should use a regular expression
// to test whether the input is valid, and if so, return the valid
// data (which may be transformed from the original user input).
// Throw a ValidationException if your validation fails.
public class Validate {
    public static void USZipCode(string userInput) {
        string pattern = @"
^       # nothing before this!
\d{5}   # main zip code body
(       # optional zip+4 extension
 -?     # may or may not have a hyphen
 \d{4}  # here's the +4 part
)?$     # nothing after this!";
        if (!Regex.IsMatch(userInput, pattern, RegexOptions.IgnorePatternWhitespace)) {
            throw new ValidationException("Malformed zip code");
        }
    }
    public static void USPhoneNumber(string userInput) {
        string pattern = @"^[01]?[- .]?(\([2-9]\d{2}\)|[2-9]\d{2})[- .]?\d{3}[- .]?\d{4}$";
        if (!Regex.IsMatch(userInput, pattern, RegexOptions.IgnorePatternWhitespace)) {
            throw new ValidationException("Malformed phone number");
        }
    }
    public static void Currency(string userInput) {
        string pattern = @"^\d+(\.\d\d)?$";
        if (!Regex.IsMatch(userInput, pattern, RegexOptions.IgnorePatternWhitespace)) {
            throw new ValidationException("Malformed currency");
        }
    }
    public static void EmailAddress(string userInput) {
        string pattern = @"^([0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*@([0-9a-zA-Z][-\w]*[0-9a-zA-Z]\.)+[a-zA-Z]{2,9})$";
        if (!Regex.IsMatch(userInput, pattern, RegexOptions.IgnorePatternWhitespace)) {
            throw new ValidationException("Malformed email address");
        }
    }
}

public class ValidationException : Exception {
    public ValidationException(string msg) : base(msg) { }
}

// The main program exercises your validation logic,
// writing any failures to stdout. If you have zero failures,
// you'll see no output at all!
class Program {
    static void Main(string[] args) {
        string testMethod = "USZipCode";
        Test.Good("12345", testMethod);
        Test.Bad("1234", testMethod);
        Test.Good("12345-1234", testMethod);
        Test.Good("123451234", testMethod);
        Test.Bad("ABCDE", testMethod);
        Test.Bad("", testMethod);

        testMethod = "USPhoneNumber";
        Test.Good("4255550123", testMethod);
        Test.Good("(425) 555-0123", testMethod);
        Test.Good("425-555-0123", testMethod);
        Test.Good("425 555 0123", testMethod);
        Test.Good("1-425-555-0123", testMethod);
        Test.Bad("425555012", testMethod);
        Test.Bad("555-0123", testMethod);
        Test.Bad("", testMethod);

        testMethod = "Currency";
        Test.Good("1.00", testMethod);
        Test.Good("3343", testMethod);
        Test.Bad("1234.567", testMethod);
        Test.Bad("", testMethod);

        testMethod = "EmailAddress";
        Test.Good("alice@foo.com", testMethod);
        Test.Good("alice@foo.co.uk", testMethod);
        Test.Bad("alice@bar.c", testMethod);
        Test.Bad("", testMethod);
        Test.Bad("'drop table customers--", testMethod);

    }
}
