using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace RegexBench {
    public partial class MainForm : Form {
        public MainForm() {
            InitializeComponent();

            computeRegexOptions();
        }

        private void automatch() {
            if (chkAutoMatch.Checked) {
                match();
            }
        }

        private void match() {
            string pattern = txtPattern.Text;
            string input = txtInput.Text;

            if (pattern.Length > 0) {
                try {
                    Regex regex = new Regex(pattern, regexOptions);
                    Match m = regex.Match(input);
                    if (m.Success) {
                        StringBuilder detail = new StringBuilder();
                        for (int i = 0; i < m.Groups.Count; ++i) {
                            detail.AppendFormat("Group[{0}]: {1}", i, m.Groups[i].Value);
                            detail.AppendLine();
                        }
                        setResult(true, detail.ToString());
                    }
                    else setResult(m.Success);
                }
                catch (Exception x) {
                    setResult(x);
                }
            }
            else {
                setResult(false, "Regular expression pattern is empty.");
            }
        }

        RegexOptions regexOptions;

        private void computeRegexOptions() {
            CheckBox[] options = {
                chkIgnoreCase,
                chkSingleline,
                chkCompiled,
                chkCultureInvariant,
                chkECMAScript,
                chkExplicitCapture,
                chkIgnorePatternWhitespace,
                chkMultiline,
                chkRightToLeft,
            };
            regexOptions = RegexOptions.None;
            foreach (CheckBox option in options) {
                if (option.Checked) {
                    regexOptions |= (RegexOptions)System.Enum.Parse(typeof(RegexOptions), option.Text);
                }
            }
        }

        private void setResult(Exception x) {
            txtResult.Text = string.Format("Exception: {0}", x.Message);
            pnlResult.BackColor = Color.Red;
        }

        private void setResult(bool match, string detail) {
            txtResult.Text = detail;
            pnlResult.BackColor = match ? Color.Green : Color.Yellow;
        }

        private void setResult(bool match) {
            txtResult.Text = match ? "Match." :
                                     "No match.";
            pnlResult.BackColor = match ? Color.Green : Color.Yellow;
        }

        private void btnMatch_Click(object sender, EventArgs e) {
            match();
        }

        private void OnRegexOptionsChange(object sender, EventArgs e) {
            computeRegexOptions();
            automatch();
        }

        private void txtRegex_TextChanged(object sender, EventArgs e) {
            automatch();
        }

        private void txtInput_TextChanged(object sender, EventArgs e) {
            automatch();
        }
    }
}